# Libraries ----------------------------------------------------------------------------------------
library(rio)
library(tidyverse)
library(httr)
library(RCurl)

# define base URL, username and password (if required)
api_url <- "https://gitlab.fiocruz.br/api/v4/projects/345/repository/commits?ref_name=master&all=true&per_page=100&page=1" # per_page=100
username <- "*****"
password <- "*****"

login <- GET(
  api_url, 
  # authenticate(username, password)
)

# testing connection
if(login$status == 200L) {
  print("Logged in successfully!")
} else {
  stop("Could not login")
}

# list of commits
infog_raw <- jsonlite::fromJSON(api_url, flatten=TRUE)

# commit's hashes/ids
infog_ids <- infog_raw |> select(id)

# constructing url for later download
infog_url1 <- "https://gitlab.fiocruz.br/marcelo.gomes/infogripe/-/raw/"

infog_url2 <- "/Dados/InfoGripe/obitos_semanais_fx_etaria_virus_sem_filtro_febre.csv"

# pulling the full url and commit hash
infog_full <- infog_ids |> 
  mutate(
    url = paste0(infog_url1, id, infog_url2)
  )

# pulling urls
infog_commits <- infog_full |> 
  select(url) |> 
  pull()

# loading a list of datasets
infog_files_raw <- purrr::map(
  .x = seq_along(infog_commits),
  ~ getURL(
    infog_commits[.x] |> 
      as.character()
  ) |> 
    read_csv2()
)

# allocating each dataset's creation date and commit hash
infog_files <- set_names(
  infog_files_raw, 
  infog_full |> 
    select(id) |> 
    pull()
) |> 
  list_rbind(
    names_to = "commit"
  ) |> 
  left_join(
    infog_raw |> 
      select(id, committed_date),
    by = c("commit" = "id")
  ) |> 
  mutate(
    committed_date = as.Date(committed_date),
    committed_epi_yw = paste0(
      year(committed_date), 
      ifelse(
        nchar(epiweek(committed_date)) == 1, 
        paste0(0, epiweek(committed_date)), 
        epiweek(committed_date)
      )
    ),
    epi_yw = paste0(
      epiyear, 
      ifelse(
        nchar(epiweek) == 1, 
        paste0(0, epiweek), 
        epiweek
      )
    )
  ) |> 
  relocate(
    committed_date, committed_epi_yw, commit, epi_yw
  ) |> 
  arrange(desc(committed_date)) |> 
  filter(epiyear >= 2020)

# differences in counts between each commit
last_ews <- infog_files |> select(epi_yw) |> arrange(desc(epi_yw)) |> unique() |> pull()

cols_sg_ufs <- infog_files |> filter(epi_yw == last_ews[1]) |> select(SG_UF_NOT) |> pull()

cols_fx_etaria <- infog_files |> filter(epi_yw == last_ews[1]) |> select(fx_etaria) |> pull()

# calculating the difference between current epiweek and last epiweek for the last 100 commits
last_diff <- list()

for (i in 1:length(last_ews)) {
  if (i <= length(last_ews)) {
    last_diff[[i]] = infog_files |> filter(epi_yw == last_ews[i]) |> filter(committed_date == min(committed_date)) |> select(where(is.numeric)) - 
      infog_files |> filter(epi_yw == last_ews[i + 1]) |> filter(committed_date == min(committed_date)) |> select(where(is.numeric))
    
    last_diff[[i]] = last_diff[[i]] |> 
      mutate(
        SG_UF_NOT = cols_sg_ufs,
        fx_etaria = cols_fx_etaria,
        calculo = paste0(last_ews[i], " - ",  last_ews[i + 1])
      ) |> 
      relocate(
        fx_etaria, .after = SG_UF_NOT
      ) |> 
      relocate(calculo) |> 
      select(-c(epiyear:last_col()))
  }
}

df_diffs <- last_diff |> 
  list_rbind()

df_diffs |> 
  group_by(fx_etaria) |> 
  summarise(
    mean = mean(SRAG),
    median = median(SRAG),
    cases_removed = min(SRAG),
    cases_added = max(SRAG)
  )

