library(readr)
library(rio)
library(lubridate)

# WHO's flunet global updated dataset
url <- "https://xmart-api-public.who.int/FLUMART/VIW_FNT?$format=csv"

# Getting only Brazil
flunet <- read_csv(url) |> 
  filter(COUNTRY_CODE == "BRA") |> 
  arrange(ISO_WEEKSTARTDATE)

export(
  flunet, 
  paste0("flunet_full_bra_", max(flunet$ISOYW, na.rm = TRUE), ".csv")
)

# getting only weekly received/processed specimens
flunet_specimens_bra <- flunet |> 
  filter(MMWR_YEAR %in% seq(year(Sys.Date()) - 3, year(Sys.Date()))) |> 
  pivot_wider(id_cols = ISOYW, names_from = ORIGIN_SOURCE, values_from = c(SPEC_PROCESSED_NB, SPEC_RECEIVED_NB))

export(
  flunet_specimens_bra, 
  paste0("flunet_specimens_bra_", max(flunet$ISOYW, na.rm = TRUE), ".csv")
)
